local _G = GLOBAL
local APPI = AddPrefabPostInit

PrefabFiles = {"interior_turfs_willow"}

local regular = "levels/textures/ground_noise_checkeredlawn.tex"
local volcanic = "levels/textures/ground_lava_rock.tex"

local function check(ground, inst, point, range)
	local interior = _G.GetWorld().components.interiorspawner
	local current = interior and interior.current_interior
	if not current or current.floortexture ~= ground then return false end 
	
	range = range or 2
	for _, ent in pairs(_G.TheSim:FindEntities(point.x, point.y, point.z, 4, nil, {'NOBLOCK', 'player', 'FX'})) do 
		if ent ~= inst and ent:IsValid() and ent.entity:IsVisible() and not ent.components.placer and not ent.parent then
			if _G.distsq(ent:GetPosition(), point) < range * range then return false end 
		end 
	end 
	return true
end 

local VOLCANICS = {
	dug_coffeebush = true, 
	dug_elephantcactus = true
}

AddPrefabPostInitAny(function(inst)
	local deployable = inst.components.deployable 
	if not deployable or not inst:HasTag"plant" then return end 
	
	local test = deployable.test
	deployable.test = function(inst, point, ...)
		local ground = VOLCANICS[inst.prefab] and volcanic or regular
		return test(inst, point, ...) or check(ground, inst, point, deployable.min_spacing)
	end 
end)

for prefab, _ in pairs{
	slow_farmplot_placer = true, 
	fast_farmplot_placer = true, 
	clawpalmtree_sapling_placer = true
} do 
	APPI(prefab, function(inst)
		local placeTestFn = inst.components.placer.placeTestFn
		inst.components.placer.placeTestFn = function(inst, point, ...)
			return placeTestFn(inst, point, ...) or check(regular, inst, point, 3.2)
		end
	end)
end 

local function turf(prefab, price, image, name, desc)
	local recipe = _G.Recipe(prefab,
		{_G.Ingredient("oinc", price)}, 
		_G.RENO_RECIPETABS.FLOORING, 
		_G.TECH.HOME_TWO, 
		_G.RECIPE_GAME_TYPE.COMMON, 
		nil, nil, true
	)
	recipe.image = image
	_G.STRINGS.NAMES[string.upper(prefab)] = name
	_G.STRINGS.RECIPE_DESC[string.upper(prefab)] = desc
end 

turf("interior_floor_cultivated", 100, "turf_lawn.tex", "Lawn Flooring", "Farm indoors!")
turf("interior_floor_volcanic", 250, "turf_volcano.tex", "Volcanic Flooring", "Recreate a volcanic enviroment.")